#lang racket/base
;; (Temporary) example client and server

(require racket/tcp)
(require racket/pretty)
(require racket/match)
(require racket/class)
(require racket/port)

(require "conversation.rkt")
(require "ssh-numbers.rkt")
(require "ssh-session.rkt")
(require "standard-thread.rkt")

(define (t-client)
  (let-values (((i o) (tcp-connect "localhost"
				   2323
				   ;;22
				   )))
    (let ((api (ssh-session 'client i o)))
      (printf "Got API ~v\n" api)
      (semaphore-wait (make-semaphore 0)))))

(define (repl-channel-main oob-ch in out)
  (let loop ()
    (sync (handle-evt oob-ch
		      (match-lambda
		       (`(notify ,type ,data)
			(log-info (format "repl-channel: notify ~v ~v" type data))
			(loop))
		       (`(request (#"shell" ,_) ,k)
			(k 'ok)
			(loop))
		       (`(request ,other ,k)
			(log-info (format "repl-channel: request ~v" other))
			(k 'error)
			(loop))))
	  (handle-evt (read-line-evt in 'any)
		      (lambda (line)
			(if (eof-object? line)
			    (begin (fprintf out "Goodbye\n" out)
				   'done)
			    (begin (fprintf out "You said ~s\n" line)
				   (loop))))))))

(define (t-server)
  (define s (tcp-listen 2322 4 #t "localhost"))
  (printf "Accepting...\n")
  (let loop ()
    (let-values (((i o) (tcp-accept s)))
      (standard-thread
       (lambda ()
	 (simple-ssh-server (ssh-session 'server i o)
			    (lambda (username channel-type extra-request-data state)
			      (match channel-type
				(#"session"
				 (values `(ok ,repl-channel-main #"")
					 state))
				(else
				 (values `(error ,SSH_OPEN_UNKNOWN_CHANNEL_TYPE
						 "Unknown channel type")
					 state))))
			    'no-state)))
      (loop))))

(if (getenv "clientmode")
    (t-client)
    (t-server))
